/* see LICENSE file for copyright and license details. */
#ifndef COBBLE_PACKET_H
#define COBBLE_PACKET_H

#include <stdint.h>
#include <stdlib.h>
#include <stdbool.h>

#include <cobble/connection.h>

/* enum representing the 1.14.3 protocol */
enum packet_type {
	INVALID,
	HANDSHAKE_SERVER_HANDSHAKE,
	PLAY_CLIENT_SPAWN_OBJECT,
	PLAY_CLIENT_SPAWN_EXPERIENCE_ORB,
	PLAY_CLIENT_SPAWN_GLOBAL_ENTITY,
	PLAY_CLIENT_SPAWN_MOB,
	PLAY_CLIENT_SPAWN_PAINTING,
	PLAY_CLIENT_SPAWN_PLAYER,
	PLAY_CLIENT_ANIMATION,
	PLAY_CLIENT_STATISTICS,
	PLAY_CLIENT_BLOCK_BREAK_ANIMATION,
	PLAY_CLIENT_UPDATE_BLOCK_ENTITY,
	PLAY_CLIENT_BLOCK_ACTION,
	PLAY_CLIENT_BLOCK_CHANGE,
	PLAY_CLIENT_BOSS_BAR,
	PLAY_CLIENT_SERVER_DIFFICULTY,
	PLAY_CLIENT_CHAT_MESSAGE,
	PLAY_CLIENT_MULTI_BLOCK_CHANGE,
	PLAY_CLIENT_TAB_COMPLETE,
	PLAY_CLIENT_DECLARE_COMMANDS,
	PLAY_CLIENT_CONFIRM_TRANSACTION,
	PLAY_CLIENT_CLOSE_WINDOW,
	PLAY_CLIENT_WINDOW_ITEMS,
	PLAY_CLIENT_WINDOW_PROPERTY,
	PLAY_CLIENT_SET_SLOT,
	PLAY_CLIENT_SET_COOLDOWN,
	PLAY_CLIENT_PLUGIN_MESSAGE,
	PLAY_CLIENT_NAMED_SOUND_EFFECT,
	PLAY_CLIENT_DISCONNECT,
	PLAY_CLIENT_ENTITY_STATUS,
	PLAY_CLIENT_EXPLOSION,
	PLAY_CLIENT_UNLOAD_CHUNK,
	PLAY_CLIENT_CHANGE_GAME_STATE,
	PLAY_CLIENT_OPEN_HORSE_WINDOW,
	PLAY_CLIENT_KEEP_ALIVE,
	PLAY_CLIENT_CHUNK_DATA,
	PLAY_CLIENT_EFFECT,
	PLAY_CLIENT_PARTICLE,
	PLAY_CLIENT_UPDATE_LIGHT,
	PLAY_CLIENT_JOIN_GAME,
	PLAY_CLIENT_MAP_DATA,
	PLAY_CLIENT_TRADE_LIST,
	PLAY_CLIENT_ENTITY_RELATIVE_MOVE,
	PLAY_CLIENT_ENTITY_LOOK_AND_RELATIVE_MOVE,
	PLAY_CLIENT_ENTITY_LOOK,
	PLAY_CLIENT_ENTITY,
	PLAY_CLIENT_VEHICLE_MOVE,
	PLAY_CLIENT_OPEN_BOOK,
	PLAY_CLIENT_OPEN_WINDOW,
	PLAY_CLIENT_OPEN_SIGN_EDITOR,
	PLAY_CLIENT_CRAFT_RECIPE_RESPONSE,
	PLAY_CLIENT_PlAYER_ABILITIES,
	PLAY_CLIENT_COMBAT_EVENT,
	PLAY_CLIENT_PLAYER_INFO,
	PLAY_CLIENT_FACE_PLAYER,
	PLAY_CLIENT_PLAYER_POSITION_AND_LOOK,
	PLAY_CLIENT_UNLOCK_RECIPES,
	PLAY_CLIENT_DESTROY_ENTITIES,
	PLAY_CLIENT_REMOVE_ENTITY_EFFECT,
	PLAY_CLIENT_RESOURCE_PACK_SEND,
	PLAY_CLIENT_RESPAWN,
	PLAY_CLIENT_ENTITY_HEAD_LOOK,
	PLAY_CLIENT_SELECT_ADVANCEMENT_TAB,
	PLAY_CLIENT_WORLD_BORDER,
	PLAY_CLIENT_CAMERA,
	PLAY_CLIENT_HELD_ITEM_CHANGE,
	PLAY_CLIENT_UPDATE_VIEW_POSITION,
	PLAY_CLIENT_UPDATE_VIEW_DISTANCE,
	PLAY_CLIENT_DISPLAY_SCOREBOARD,
	PLAY_CLIENT_ENTITY_METADATA,
	PLAY_CLIENT_ATTACH_ENTITY,
	PLAY_CLIENT_ENTITY_VELOCITY,
	PLAY_CLIENT_ENTITY_EQUIPMENT,
	PLAY_CLIENT_SET_EXPERIENCE,
	PLAY_CLIENT_UPDATE_HEALTH,
	PLAY_CLIENT_SCOREBOARD_OBJECTIVE,
	PLAY_CLIENT_SET_PASSENGERS,
	PLAY_CLIENT_TEAMS,
	PLAY_CLIENT_UPDATE_SCORE,
	PLAY_CLIENT_SPAWN_POSITION,
	PLAY_CLIENT_TIME_UPDATE,
	PLAY_CLIENT_TITLE,
	PLAY_CLIENT_ENTITY_SOUND_EFFECT,
	PLAY_CLIENT_SOUND_EFFECT,
	PLAY_CLIENT_STOP_SOUND,
	PLAY_CLIENT_PLAYER_LIST_HEADER_AND_FOOTER,
	PLAY_CLIENT_NBT_QUERY_RESPONSE,
	PLAY_CLIENT_COLLECT_ITEM,
	PLAY_CLIENT_ENTITY_TELEPORT,
	PLAY_CLIENT_ADVANCEMENTS,
	PLAY_CLIENT_ENTITY_PROPERTIES,
	PLAY_CLIENT_ENTITY_EFFECT,
	PLAY_CLIENT_DECLARE_RECIPES,
	PLAY_CLIENT_TAGS,
	PLAY_CLIENT_ACKNOWLEDGE_PLAYER_DIGGING,
	PLAY_SERVER_TELEPORT_CONFIRM,
	PLAY_SERVER_QUERY_BLOCK_NBT,
	PLAY_SERVER_SET_DIFFICULTY,
	PLAY_SERVER_CHAT_MESSAGE,
	PLAY_SERVER_CLIENT_STATUS,
	PLAY_SERVER_CLIENT_SETTINGS,
	PLAY_SERVER_TAB_COMPLETE,
	PLAY_SERVER_CONFIRM_TRANSACTION,
	PLAY_SERVER_CLICK_WINDOW_BUTTON,
	PLAY_SERVER_CLICK_WINDOW,
	PLAY_SERVER_CLOSE_WINDOW,
	PLAY_SERVER_PLUGIN_MESSAGE,
	PLAY_SERVER_EDIT_BOOK,
	PLAY_SERVER_QUERY_ENTITY_NBT,
	PLAY_SERVER_USE_ENTITY,
	PLAY_SERVER_KEEP_ALIVE,
	PLAY_SERVER_LOCK_DIFFICULTY,
	PLAY_SERVER_PLAYER_POSITION,
	PLAY_SERVER_PLAYER_POSITION_AND_LOOK,
	PLAY_SERVER_PLAYER_LOOK,
	PLAY_SERVER_PLAYER,
	PLAY_SERVER_VEHICLE_MOVE,
	PLAY_SERVER_STEER_BOAT,
	PLAY_SERVER_PICK_ITEM,
	PLAY_SERVER_CRAFT_RECIPE_REQUEST,
	PLAY_SERVER_PLAYER_ABILITIES,
	PLAY_SERVER_PLAYER_DIGGING,
	PLAY_SERVER_ENTITY_ACTION,
	PLAY_SERVER_STEER_VEHICLE,
	PLAY_SERVER_RECIPE_BOOK_DATA,
	PLAY_SERVER_NAME_ITEM,
	PLAY_SERVER_RESOURCE_PACK_STATUS,
	PLAY_SERVER_ADVANCEMENT_TAB,
	PLAY_SERVER_SELECT_TRADE,
	PLAY_SERVER_SET_BEACON_EFFECT,
	PLAY_SERVER_HELD_ITEM_CHANGE,
	PLAY_SERVER_UPDATE_COMMAND_BLOCK,
	PLAY_SERVER_UPDATE_COMMAND_BLOCK_MINECART,
	PLAY_SERVER_CREATIVE_INVENTORY_ACTION,
	PLAY_SERVER_UPDATE_JIGSAW_BLOCK,
	PLAY_SERVER_UPDATE_STRUCTURE_BLOCK,
	PLAY_SERVER_UPDATE_SIGN,
	PLAY_SERVER_ANIMATION,
	PLAY_SERVER_SPECTATE,
	PLAY_SERVER_PLAYER_BLOCK_PLACEMENT,
	PLAY_SERVER_USE_ITEM,
	STATUS_CLIENT_RESPONSE,
	STATUS_CLIENT_PONG,
	STATUS_SERVER_REQUEST,
	STATUS_SERVER_PING,
	LOGIN_CLIENT_DISCONNECT,
	LOGIN_CLIENT_ENCRYPTION_REQUEST,
	LOGIN_CLIENT_LOGIN_SUCCESS,
	LOGIN_CLIENT_SET_COMPRESSION,
	LOGIN_CLIENT_LOGIN_PLUGIN_REQUEST,
	LOGIN_SERVER_LOGIN_START,
	LOGIN_SERVER_ENCRYPTION_RESPONSE,
	LOGIN_SERVER_LOGIN_PLUGIN_RESPONSE
};

/* generic packet type */
struct packet {
	int32_t id;
	enum packet_type type;
	void *data;
};

struct packet* packet_new(enum packet_type, void*);
bool packet_read(struct connection*, struct packet*, uint8_t*, size_t);
bool packet_write(struct connection*, struct packet*);
size_t packet_size(struct packet*);
void packet_destroy(struct packet*);
void packet_handle(struct connection*, struct packet*);

#endif
