/* see LICENSE file for copyright and license details. */
#ifndef COBBLE_PACKET_IMPL_H
#define COBBLE_PACKET_IMPL_H
#include <cobble/player.h>
#include <cobble/world.h>

/* HANDSHAKE_SERVER_HANDSHAKE */
struct packet_handshake {
	int32_t protocol;
	char *server_address;
	uint16_t server_port;
	int32_t next_state;
};

/* PLAY_CLIENT_JOIN_GAME */
struct packet_join_game {
	int32_t entity_id;
	enum gamemode gamemode;
	enum dimension dimension;
	enum difficulty difficulty;
	/* this field is ignored */	
	uint8_t max_players; 
	char *level_type;
	bool reduced_debug_info;
};

/* STATUS_CLIENT_RESPONSE */
struct packet_response {
	char *response;
};

/* STATUS_CLIENT_PONG */
struct packet_pong {
	int64_t payload;
};

/* STATUS_SERVER_PING */
struct packet_ping {
	int64_t payload;
};

/* LOGIN_CLIENT_DISCONNECT */
struct packet_login_disconnect {
	char *reason;
};

/* LOGIN_CLIENT_ENCRYPTION_REQUEST */
struct packet_encryption_request {
	char *server_id;
	int32_t public_key_len;
	uint8_t *public_key;
	int32_t verify_token_len;
	uint8_t verify_token;
};

/* LOGIN_CLIENT_LOGIN_SUCCESS */
struct packet_login_success {
	char *uuid;
	char *username;
};

/* LOGIN_CLIENT_SET_COMPRESSION */
struct packet_set_compression {
	int32_t threshold;
};

/* LOGIN_SERVER_LOGIN_START */
struct packet_login_start {
	char *username;
};

/* LOGIN_SERVER_ENCRYPTION_RESPONSE */
struct packet_encryption_response {
	int32_t shared_secret_len;
	uint8_t *shared_secret;
	int32_t verify_token_len;
	uint8_t *verify_token;
};

/* HANDSHAKE_SERVER_HANDSHAKE */
bool packet_handshake_read(struct connection*, struct packet*, uint8_t *data, size_t data_len);
void packet_handshake_destroy(struct packet*);
void packet_handshake_handle(struct connection*, struct packet*);

/* PLAY_CLIENT_JOIN_GAME */
bool packet_join_game_read(struct connection*, struct packet*, uint8_t *data, size_t data_len);
void packet_join_game_destroy(struct packet*);
void packet_join_game_handle(struct connection*, struct packet*);

/* STATUS_CLIENT_RESPONSE */
bool packet_response_write(struct connection*, struct packet*, uint8_t *buf, size_t data_len);
size_t packet_response_size(struct packet*);
void packet_response_destroy(struct packet*);

/* STATUS_CLIENT_PONG */
bool packet_pong_write(struct connection*, struct packet*, uint8_t *buf, size_t data_len);
size_t packet_pong_size(struct packet*);

/* STATUS_SERVER_REQUEST */
bool packet_request_read(struct connection*, struct packet*, uint8_t *data, size_t data_len);
void packet_request_handle(struct connection*, struct packet*);

/* STATUS_SERVER_PING */
bool packet_ping_read(struct connection*, struct packet*, uint8_t *data, size_t data_len);
void packet_ping_handle(struct connection*, struct packet*);

/* LOGIN_CLIENT_LOGIN_SUCCESS */
bool packet_login_success_write(struct connection*, struct packet*, uint8_t *buf, size_t data_len);
size_t packet_login_success_size(struct packet*);
void packet_login_success_destroy(struct packet*);

/* LOGIN_SERVER_LOGIN_START */
bool packet_login_start_read(struct connection*, struct packet*, uint8_t *data, size_t data_len);
void packet_login_start_destroy(struct packet*);
void packet_login_start_handle(struct connection*, struct packet*);

#endif
