/* see LICENSE file for copyright and license details. */
#include <stdio.h>
#include <pthread.h>

#include <rxi/log.h>
#include <rxi/dyad.h>

#include <cobble/connection.h>
#include <cobble/packet.h>
#include <cobble/protocol.h>
#include <cobble/server.h>
#include <cobble/util.h>

#include <../config.h>

/* called when a connection receives data */
static void 
connection_on_data(dyad_Event *e) 
{
	struct connection *conn;
	struct packet *packet;
	size_t len_size;
	int len;
	
	conn = (struct connection*) e->udata;
	len_size = read_varint(&len, e->data, e->size);

	if (len_size == 0 || e->size < len) {
		/* if this is reached, we didn't get the full packet from the client */
		/* TODO actually print out information about client */
		log_warn("got an invalid packet from client {}");
		return;
	}

	packet = SAFE_MALLOC(sizeof(struct packet));
	packet_read(conn, packet, e->data + len_size, len);
	packet_handle(conn, packet);
	packet_destroy(packet);
}

static void
connection_on_close(dyad_Event *e)
{
	struct connection *conn;
	
	conn = (struct connection*) e->data;
	free(conn);
}

/* called when the server gets a new connection */
static void
server_on_accept(dyad_Event *e)
{
	struct connection *conn;
	struct server *server;

	server = (struct server*) e->udata;

	conn = SAFE_MALLOC(sizeof(struct connection));
	conn->dyad_stream = e->remote;
	conn->server = server;
	conn->protocol_state = HANDSHAKE;

	dyad_addListener(e->remote, DYAD_EVENT_DATA, connection_on_data, conn);
	dyad_addListener(e->remote, DYAD_EVENT_CLOSE, connection_on_close, conn);
	log_debug("Got a connection");
}

void
connection_thread(void *args) 
{
	struct server *server;

	server = (struct server*) args;
	dyad_addListener(server->dyad_stream, DYAD_EVENT_ACCEPT, server_on_accept, server);
	/* TODO make the backlog count (last arg) configurable or something */
	dyad_listenEx(server->dyad_stream, COBBLE_SERVER_IP, COBBLE_SERVER_PORT, 128);
	
	/* main dyad loop */
	while (1) {
		dyad_update();
	}
}
